﻿using SoapEndpoint.Service.Flows.SbEnSk.v1.LogbookService;
using SoapEndpoint.Service.Proxy.ServiceProxies.Systeem.v2.ErrorMessage;
using SoapEndpoint.Tester.Scenario.Flow.SbEnSk;

namespace SoapEndpoint.Tester.Scenario.Implementations.SbEnSk;

public class VervallenBerichtScenario : SbEnSkScenarioBase
{
    public override string Name => "Vervallen bericht scenario";

    protected override async Task ExecuteAsync()
    {
        await ReactieopVervallenKennisgevingDoorSaldoverzoek();

        await ReactieOpSaldoverzoekNaVervallenVanKennisgeving();

        await ReactieOpEersteBetaalvoorstelNaVervallenDoorTweedeBetaalvoorstel();

        await ReactieOpTweedeBetaalvoorstelNaVervallenVanEersteBetaalvoorstel();

        await Beeindiging();

        await VersturenShvBerichtZonderSeBerichtTeBevestigen();

        await ReactieOpBevestigdeEnVervallenKennisgevingNaVersturenSaldoverzoek();

        await ReactieOpVervallenHeroverwegingNaVersturenTweedeBetaalvoorstel();

        await ReactieOpEersteBevestigdeBetaalvoorstelNaVervallenDoorTweedeBetaalvoorstel();
    }

    private async Task ReactieopVervallenKennisgevingDoorSaldoverzoek()
    {
        StartNewScenario("Reactie op vervallen kennisgeving door saldoverzoek");

        //Send kennisgeving
        var supersededKennisgeving = await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Kennisgeving);
        
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        //Send a saldoverzoek without confirming the kennisgeving message. It should cause the kennisgeving message to be superseded
        await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Saldoverzoek);

        // If this scenario fails, it might be because we did not wait long enough for the messages to be processed. 
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Saldoverzoek);

        //Try to reply to the superseded kennisgeving message.
        await Manager.SE_SendMessageAsync(Conversation, BerichtType.SE_KennisgevingAntwoord,
            supersededKennisgeving);

        // Check that the correct error is returned.
        await Manager
            .SE_GetAndConfirmErrorMessage<ShvKnooppuntFoutbericht, BerichtType, ShvKnooppuntFoutcode>(
                Conversation, BerichtType.SYS_FoutBericht,
                ShvKnooppuntFoutcode.Berichtvervallenomdathetisopgevolgddooreennieuwbericht);
    }

    private async Task ReactieOpSaldoverzoekNaVervallenVanKennisgeving()
    {
        StartNewScenario("Reactie op saldoverzoek na vervallen van kennisgeving");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_Kennisgeving),
            new ShvFlowItem(BerichtType.SHV_Saldoverzoek, BerichtType.SE_SaldoverzoekAntwoord));
    }

    private async Task ReactieOpEersteBetaalvoorstelNaVervallenDoorTweedeBetaalvoorstel()
    {
        StartNewScenario("Reactie op eerste betaalvoorstel na vervallen door tweede betaalvoorstel");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_Saldoverzoek, BerichtType.SE_SaldoverzoekAntwoord));

        //Send betaalvoorstel
        var supersededBetaalvoorstel =
            await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Betaalvoorstel);
        
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        //Send a new betaalvoorstel without confirming the previous betaalvoorstel message. It should cause the previous betaalvoorstel message to be superseded
        await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Betaalvoorstel);

        // If this scenario fails, it might be because we did not wait long enough for the messages to be processed. 
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Betaalvoorstel);

        //Try replying to the superseded betaalvoorstel message
        await Manager.SE_SendMessageAsync(Conversation, BerichtType.SE_BetaalvoorstelAntwoordAkkoord,
            supersededBetaalvoorstel);

        // Check that the correct error is returned.
        await Manager.SE_GetAndConfirmErrorMessage<ShvKnooppuntFoutbericht, BerichtType, ShvKnooppuntFoutcode>(
            Conversation,
            BerichtType.SYS_FoutBericht,
            ShvKnooppuntFoutcode.Berichtvervallenomdathetisopgevolgddooreennieuwbericht);
    }

    private async Task ReactieOpTweedeBetaalvoorstelNaVervallenVanEersteBetaalvoorstel()
    {
        StartNewScenario("Reactie op tweede betaalvoorstel na vervallen van eerste betaalvoorstel");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_Saldoverzoek, BerichtType.SE_SaldoverzoekAntwoord),
            new ShvFlowItem(BerichtType.SHV_Betaalvoorstel));

        //Send a new betaalvoorstel without confirming the previous betaalvoorstel message. It should cause the previous betaalvoorstel message to be superseded
        var betaalVoorstel = await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Betaalvoorstel);

        // If this scenario fails, it might be because we did not wait long enough for the messages to be processed. 
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Betaalvoorstel);

        //Try replying to the superseded betaalvoorstel message
        await Manager.SE_SendMessageAsync(Conversation, BerichtType.SE_BetaalvoorstelAntwoordAkkoord,
            betaalVoorstel);
    }

    private async Task Beeindiging()
    {
        StartNewScenario("Beeindiging na bevestigde bericht");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_Saldoverzoek, BerichtType.SE_SaldoverzoekAntwoord),
            new ShvFlowItem(BerichtType.SHV_Beeindiging));

        StartNewScenario("Beeindiging na onbevestigde bericht");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_Saldoverzoek),
            new ShvFlowItem(BerichtType.SHV_Beeindiging));
    }

    private async Task VersturenShvBerichtZonderSeBerichtTeBevestigen()
    {
        StartNewScenario("Versturen SHV bericht zonder SE bericht te bevestigen");

        //Send a new betaalvoorstel without confirming the previous betaalvoorstel message. It should cause the previous betaalvoorstel message to be superseded
        await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Saldoverzoek);

        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Saldoverzoek);

        await Manager.SE_SendMessageAsync(Conversation, BerichtType.SE_SaldoverzoekAntwoord);

        //Send betaalvoorstel without confirming saldoverzoek antwoord
        await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Betaalvoorstel);

        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        // Check that the correct error is returned.
        await Manager
            .SHV_GetAndConfirmErrorMessage<ShvKnooppuntFoutbericht, BerichtType, ShvKnooppuntFoutcode>(
                Conversation, BerichtType.SYS_FoutBericht,
                ShvKnooppuntFoutcode.Berichttypeniettoegestaanvoordehuidigestaatvandeconversatie);
    }

    private async Task ReactieOpBevestigdeEnVervallenKennisgevingNaVersturenSaldoverzoek()
    {
        StartNewScenario("Reactie op saldoverzoek na vervallen van kennisgeving");

        //Send kennisgeving message
        var supersededKennisgeving = await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Kennisgeving);

        // If this scenario fails, it might be because we did not wait long enough for the messages to be processed. 
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        //Confirm the kennisgeving message
        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Kennisgeving);

        await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Saldoverzoek);

        // If this scenario fails, it might be because we did not wait long enough for the messages to be processed. 
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        //Confirm the saldoverzoek message
        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Saldoverzoek);

        //Try replying to the superseded kennisgeving message
        await Manager.SE_SendMessageAsync(Conversation, BerichtType.SE_KennisgevingAntwoord,
            supersededKennisgeving);

        // Check that the correct error is returned.
        await Manager.SE_GetAndConfirmErrorMessage<ShvKnooppuntFoutbericht, BerichtType, ShvKnooppuntFoutcode>(
            Conversation,
            BerichtType.SYS_FoutBericht,
            ShvKnooppuntFoutcode.Berichtvervallenomdathetisopgevolgddooreennieuwbericht);
    }

    private async Task ReactieOpVervallenHeroverwegingNaVersturenTweedeBetaalvoorstel()
    {
        StartNewScenario("Reactie op vervallen heroverweging na versturen tweede betaal voorstel");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_Saldoverzoek, BerichtType.SE_SaldoverzoekAntwoord),
            new ShvFlowItem(BerichtType.SHV_Betaalvoorstel, BerichtType.SE_BetaalvoorstelAntwoordNietAkkoord));

        var supersededHeroverweging =
            await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Heroverweging);

        //Confirm the heroverweging message
        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Heroverweging);

        //Send new betaalvoorstel
        await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Betaalvoorstel);

        // If this scenario fails, it might be because we did not wait long enough for the messages to be processed. 
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        //Try replying to the superseded heroverweging message
        await Manager.SE_SendMessageAsync(Conversation, BerichtType.SE_BetaalvoorstelAntwoordNietAkkoord,
            supersededHeroverweging);

        // If this scenario fails, it might be because we did not wait long enough for the messages to be processed. 
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        // Check that the correct error is returned.
        await Manager.SE_GetAndConfirmErrorMessage<ShvKnooppuntFoutbericht, BerichtType, ShvKnooppuntFoutcode>(
            Conversation,
            BerichtType.SYS_FoutBericht,
            ShvKnooppuntFoutcode.Berichtvervallenomdathetisopgevolgddooreennieuwbericht);
    }

    private async Task ReactieOpEersteBevestigdeBetaalvoorstelNaVervallenDoorTweedeBetaalvoorstel()
    {
        StartNewScenario("Reactie op eerste betaalvoorstel na vervallen door tweede betaalvoorstel");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_Saldoverzoek, BerichtType.SE_SaldoverzoekAntwoord));

        //Send betaalvoorstel
        var supersededBetaalvoorstel =
            await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Betaalvoorstel);

        //Confirm the first betaalvoorstel message
        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Betaalvoorstel);

        //Send the second betaalvoorstel
        await Manager.SHV_SendMessageAsync(Conversation, BerichtType.SHV_Betaalvoorstel);

        // If this scenario fails, it might be because we did not wait long enough for the messages to be processed. 
        await Manager.WaitForMessagesToBeProcessed(TimeSpan.FromSeconds(3));

        await Manager.SE_GetAndConfirm(Conversation, BerichtType.SHV_Betaalvoorstel);

        //Try replying to the superseded betaalvoorstel message
        await Manager.SE_SendMessageAsync(Conversation, BerichtType.SE_BetaalvoorstelAntwoordAkkoord,
            supersededBetaalvoorstel);

        // Check that the correct error is returned.
        await Manager.SE_GetAndConfirmErrorMessage<ShvKnooppuntFoutbericht, BerichtType, ShvKnooppuntFoutcode>(
            Conversation,
            BerichtType.SYS_FoutBericht,
            ShvKnooppuntFoutcode.Berichtvervallenomdathetisopgevolgddooreennieuwbericht);
    }
}