﻿using System.Security.Cryptography.X509Certificates;
using System.Text;
using SoapEndpoint.Service.Proxy.Infrastructure;
using SoapEndpoint.Service.Proxy.Models.Flows;
using SoapEndpoint.Service.Proxy.Models.Systeem;
using SoapEndpoint.Service.Proxy.ResourceItems;
using SoapEndpoint.Tester.Infrastructure;
using Environment = SoapEndpoint.Tester.Infrastructure.Environment;

namespace SoapEndpoint.Tester.Scenario;

public class ScenarioRunner
{
    public ScenarioRunner()
    {
        Output = new StringBuilder();
    }

    private IList<CertificateItem> CertificateItems { get; set; }
    public StringBuilder Output { get; }

    private void AddMessage(string message, bool success = true)
    {
        Output.AppendLine($"{message}");
    }

    public async Task<List<ScenarioResultGroup>> RunAsync(string url, params X509Certificate2[] certificates)
    {
        CertificateItems = certificates
            .Select(i => new CertificateItem(i))
            .ToList();

        var environment = new Environment(
            url, url);

        // TODO: improve this.
        var version = SystemVersions.V1_0;

        var result = await RunAsync(environment, version, CertificateItems);
        return result;
    }


    private async Task<List<ScenarioResultGroup>> RunAsync(
        Environment currentEnvironment,
        Version currentVersion,
        IList<CertificateItem> certificates)
    {
        var mapper = AutoMapperHelper.InitializeAutoMapper();

        var urlInfo = new ServiceUrlInfo(mapper);

        var scenarioResultGroups = new List<ScenarioResultGroup>();

        // TODO: Improve this;
        var scenarios = Scenarios.Items[FlowType.Schuldbemiddeling].Where(s => s.Name == "Happy flow scenario")
            .ToList();

        // TODO: Improve this;
        var addressbookCertificate = certificates.FirstOrDefault(i => i.Name == "SHV Innovadis")?.Certificate;

        //Get the address book.
        AddMessage("Fetching the address book");
        var addressBookCertificate = addressbookCertificate;
        var proxy = urlInfo.GetAddressProxy(addressBookCertificate, currentEnvironment, currentVersion);
        var organizations = await proxy.GetOrganizationsAsync(new AdresboekZoekBerichtModel());

        AddMessage($"Starting to execute {scenarios} scenarios.");

        foreach (var scenario in scenarios)
            try
            {
                var manager = new ScenarioManager(
                    scenario,
                    certificates,
                    organizations,
                    new ResourceItemManager(),
                    urlInfo,
                    AddMessage,
                    currentVersion,
                    currentEnvironment);

                var resultGroup = await manager.ExecuteScenarioAsync();
                if (resultGroup != null)
                {
                    scenarioResultGroups.Add(resultGroup);
                }
                else
                {
                    AddMessage($"Skipping scenario: {scenario.Name}", false);
                }
            }
            catch (Exception scenarioEx)
            {
                AddMessage($"Scenario failed: {scenarioEx.Message}", false);
                throw;
            }

        return scenarioResultGroups;
    }
}